# contentful-schema-diff

A tool to automatically generate a starting point for a schema migration
between two Contentful spaces

`npm install --global contentful-schema-diff`

usage:
```bash
contentful-schema-diff --from <export file or space> --to <export file or space>

Options:
  --help       Show help                                               [boolean]
  --version    Show version number                                     [boolean]
  --from, -f   A contentful export file from the space that needs to be migrated
                                                                      [required]
  --to, -t     A contentful export file from the space containing the newest
               versions of the content types                          [required]
  --out, -o    The output directory in which to place the migration
  --token, -a  A Contentful management token to download content types from a
               space
  --one-file   Write all the migrations in a single file
```

Note: "from" indicates the space with the *old versions* of the content types.
You will be generating a migration *from* the state of the current production space
*to* the state of your dev space.  Thus 'from = production' and 'to = dev'.

The tool works either on Contentful export files, or by directly downloading content
from a space.

## Method 1: download content types from the space directly

```bash
$ contentful-schema-diff --from <from space> --to <to space> --token <my contentful management token>
```

## Method 2: Export the files first

```bash
$ contentful-export --space-id <from space> --management-token $CONTENTFUL_MANAGEMENT_TOKEN \
  --skip-content --skip-roles --skip-webhooks


$ contentful-export --space-id <to space> --management-token $CONTENTFUL_MANAGEMENT_TOKEN \
  --skip-content --skip-roles --skip-webhooks
```

Alternately this could also used to generate a migration between two environments

```bash
$ contentful-export --space-id <space> --management-token $CONTENTFUL_MANAGEMENT_TOKEN \
  --skip-content --skip-roles --skip-webhooks


$ contentful-export --space-id <space> --management-token $CONTENTFUL_MANAGEMENT_TOKEN \
  --skip-content --skip-roles --skip-webhooks --environment-id=dev
```

After you have downloaded the files, run the contentful-schema-diff tool:

```bash
$ contentful-schema-diff --from contentful-export-<from space>-<date>.json \
  --to contentful-export-<to space>-<date>.json
```

## Results

One or many new typescript file(s) will be created.  They will include many of the migrations
necessary to transition the "from" space to have the same structure as the "to"
space.  Where it couldn't figure out how to do the migration, it has left
a code comment with the diff of the fields for that content type.

example:

```ts
import Migration from 'contentful-migration-cli'

// Generated by contentful-schema-diff
// from fixtures/contentful-export-<from-space>-2018-04-11T17-01-56.json
// to   fixtures/contentful-export-<to-space>-2018-04-11T17-04-29.json
export = function (migration: Migration) {

  var menu = migration.editContentType('menu', { displayField: 'name',
  name: 'Menu',
  description: 'A Menu contains a number of Menu Buttons or other Menus, which will be rendered as drop-downs.' })

  /* TODO: automatically generate edits from this diff
 [
   ...
   ...
   ...
-  {
-    id: "secondaryItems"
-    name: "Secondary Items"
-    type: "Array"
-    validations: [
-    ]
-    items: {
-      type: "Link"
-      validations: [
-        {
-          linkContentType: [
-            "menu"
-            "menuButton"
-          ]
-          message: "The items must be either buttons or drop-down menus"
-        }
-      ]
-      linkType: "Entry"
-    }
-  }
   ...
 ]
 */

  var sectionContactUs = migration.createContentType('section-contact-us', { displayField: 'text',
  name: 'Section: Contact Us',
  description: '' })

  sectionContactUs.createField('text', { name: 'Text',
  type: 'Text',
  localized: false,
  required: true,
  validations: [],
  disabled: false,
  omitted: false })

}
```